Imports System
Imports System.IO
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Navigation

Imports System.Windows.Media.Media3D

Partial Public Class Window1
    Public Sub New()
        MyBase.New()

        Me.InitializeComponent()

        ' Insert code required on object creation below this point.
    End Sub

    Private Sub Window_Loaded(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)
        ' Ground.
        Dim ground_material As Material = New DiffuseMaterial(Brushes.Chocolate)
        Dim ground_mesh As MeshGeometry3D = Nothing
        MakeRectangle(TheModel, ground_mesh, ground_material, _
            New Point3D(-3, 0, -5), New Point3D(-3, 0, 5), New Point3D(3, 0, 5), New Point3D(3, 0, -5), _
            New Point(0, 0), New Point(0, 1), New Point(1, 1), New Point(1, 0))

        ' Make a bar chart.
        Dim values(,) As Double = { _
            {1.08, 2.0, 2.33, 3.0, 3.83, 4.83}, _
            {1.73, 1.78, 2.82, 3.01, 4.51, 3.98}, _
            {0.69, 1.38, 2.24, 2.94, 3.95, 3.56} _
        }
        MakeBarChart(TheModel, values, 5, -2, -4, 2, 4)
    End Sub

    ' Make a rectangle. If rect_mesh is Nothing, make a new one and add it to the model.
    ' The points p1, p2, p3, p4 should be outwardly oriented.
    ' The points u1, u2, u3, u4 give the texture coordinates for the points p1, p2, p3, p4.
    Private Sub MakeRectangle(ByVal model_group As Model3DGroup, _
     ByRef rect_mesh As MeshGeometry3D, ByVal rect_material As Material, _
     ByVal p1 As Point3D, ByVal p2 As Point3D, ByVal p3 As Point3D, ByVal p4 As Point3D, _
     ByVal u1 As Point, ByVal u2 As Point, ByVal u3 As Point, ByVal u4 As Point)

        ' Make the mesh if we must.
        If rect_mesh Is Nothing Then
            rect_mesh = New MeshGeometry3D()
            Dim new_model As New GeometryModel3D(rect_mesh, rect_material)
            model_group.Children.Add(new_model)
        End If

        ' Make the points.
        rect_mesh.Positions.Add(p1)
        rect_mesh.Positions.Add(p2)
        rect_mesh.Positions.Add(p3)
        rect_mesh.Positions.Add(p4)

        ' Set the texture coordinates.
        rect_mesh.TextureCoordinates.Add(u1)
        rect_mesh.TextureCoordinates.Add(u2)
        rect_mesh.TextureCoordinates.Add(u3)
        rect_mesh.TextureCoordinates.Add(u4)

        ' Make the triangles.
        Dim i1 As Integer = rect_mesh.Positions.Count - 4
        rect_mesh.TriangleIndices.Add(i1)
        rect_mesh.TriangleIndices.Add(i1 + 1)
        rect_mesh.TriangleIndices.Add(i1 + 2)

        rect_mesh.TriangleIndices.Add(i1)
        rect_mesh.TriangleIndices.Add(i1 + 2)
        rect_mesh.TriangleIndices.Add(i1 + 3)
    End Sub

    ' Make a 3D bar chart.
    Private Sub MakeBarChart(ByVal chart_model As Model3DGroup, ByVal values(,) As Double, ByVal max_value As Double, _
     ByVal xmin As Double, ByVal zmin As Double, ByVal xmax As Double, ByVal zmax As Double)

        ' Make colored materials.
        Dim side_brush As New LinearGradientBrush()
        side_brush.StartPoint = New Point(0, 0)
        side_brush.EndPoint = New Point(0, 1)
        side_brush.GradientStops.Add(New GradientStop(Colors.Blue, 0.0))
        side_brush.GradientStops.Add(New GradientStop(Colors.Lime, 0.25))
        side_brush.GradientStops.Add(New GradientStop(Colors.Yellow, 0.5))
        side_brush.GradientStops.Add(New GradientStop(Colors.Orange, 0.75))
        side_brush.GradientStops.Add(New GradientStop(Colors.Red, 1.0))
        Dim chart_material As Material = New DiffuseMaterial(side_brush)

        ' Make a mesh.
        Dim chart_mesh As New MeshGeometry3D()
        Dim new_model As New GeometryModel3D(chart_mesh, chart_material)
        chart_model.Children.Add(new_model)

        Dim num_x As Integer = values.GetUpperBound(0) + 1
        Dim num_z As Integer = values.GetUpperBound(1) + 1
        Dim dx As Double = (xmax - xmin) / (1.5 * num_x)
        Dim dz As Double = (zmax - zmin) / (1.5 * num_z)
        Dim x As Double = xmin
        For ix As Integer = 0 To num_x - 1
            Dim z As Double = zmin
            For iz As Integer = 0 To num_z - 1
                Dim y As Double = values(ix, iz)
                Dim umax As Double = y / max_value
                MakeRectangle(chart_model, chart_mesh, chart_material, _
                    New Point3D(x, 0, z + dz), New Point3D(x, y, z + dz), New Point3D(x, y, z), New Point3D(x, 0, z), _
                    New Point(0, 0), New Point(0, umax), New Point(1, umax), New Point(1, 0))
                MakeRectangle(chart_model, chart_mesh, chart_material, _
                    New Point3D(x + dx, 0, z + dz), New Point3D(x + dx, y, z + dz), New Point3D(x, y, z + dz), New Point3D(x, 0, z + dz), _
                    New Point(0, 0), New Point(0, umax), New Point(1, umax), New Point(1, 0))
                MakeRectangle(chart_model, chart_mesh, chart_material, _
                    New Point3D(x + dx, 0, z), New Point3D(x + dx, y, z), New Point3D(x + dx, y, z + dz), New Point3D(x + dx, 0, z + dz), _
                    New Point(0, 0), New Point(0, umax), New Point(1, umax), New Point(1, 0))
                MakeRectangle(chart_model, chart_mesh, chart_material, _
                    New Point3D(x, 0, z), New Point3D(x, y, z), New Point3D(x + dx, y, z), New Point3D(x + dx, 0, z), _
                    New Point(0, 0), New Point(0, umax), New Point(1, umax), New Point(1, 0))

                MakeRectangle(chart_model, chart_mesh, chart_material, _
                    New Point3D(x, y, z), New Point3D(x, y, z + dz), New Point3D(x + dx, y, z + dz), New Point3D(x + dx, y, z), _
                    New Point(0, umax), New Point(0, umax), New Point(1, umax), New Point(1, umax))
                MakeRectangle(chart_model, chart_mesh, chart_material, _
                    New Point3D(x, 0, z), New Point3D(x + dx, 0, z), New Point3D(x + dx, 0, z + dz), New Point3D(x, 0, z + dz), _
                    New Point(0, 0), New Point(0, 0), New Point(1, 0), New Point(1, 0))

                z += dz * 1.5
            Next iz
            x += dx * 1.5
        Next ix
    End Sub
End Class
