﻿Imports System
Imports System.IO
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Navigation

Imports System.Windows.Media.Media3D
Imports System.Math

Partial Public Class Window1
	Public Sub New()
		MyBase.New()

		Me.InitializeComponent()

		' Insert code required on object creation below this point.
	End Sub

    Private Sub Window_Loaded(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)
        ' Make the surface.
        MakeSurface(TheModel, AddressOf Splash, -4, -4, 4, 4, 0.1, 0.1, -1, 1)

        ' Display a title label.
        MakeTitle(TheModel)
    End Sub

    Private Delegate Function FunctionOfXZ(ByVal x As Double, ByVal z As Double) As Double

    ' Make a 3D surface.
    Private Sub MakeSurface(ByVal surface_model As Model3DGroup, ByVal F As FunctionOfXZ, _
     ByVal xmin As Double, ByVal zmin As Double, ByVal xmax As Double, ByVal zmax As Double, _
     ByVal dx As Double, ByVal dz As Double, ByVal ymin As Double, ByVal ymax As Double)

        ' Make colored materials.
        Dim surface_brush As New LinearGradientBrush()
        surface_brush.StartPoint = New Point(0, 0)
        surface_brush.EndPoint = New Point(0, 1)
        surface_brush.GradientStops.Add(New GradientStop(Colors.Blue, 0.0))
        surface_brush.GradientStops.Add(New GradientStop(Colors.Lime, 0.25))
        surface_brush.GradientStops.Add(New GradientStop(Colors.Yellow, 0.5))
        surface_brush.GradientStops.Add(New GradientStop(Colors.Orange, 0.75))
        surface_brush.GradientStops.Add(New GradientStop(Colors.Red, 1.0))
        Dim surface_material As Material = New DiffuseMaterial(surface_brush)

        ' Make a mesh.
        Dim surface_mesh As New MeshGeometry3D()
        Dim new_model As New GeometryModel3D(surface_mesh, surface_material)
        surface_model.Children.Add(new_model)

        Dim dy As Double = ymax - ymin
        Dim num_x As Integer = CInt((xmax - xmin) / dx)
        Dim num_z As Integer = CInt((zmax - zmin) / dz)
        Dim x As Double = xmin
        For ix As Integer = 0 To num_x - 2
            Dim z As Double = zmin
            For iz As Integer = 0 To num_z - 2
                ' Make the points.
                Dim i1 As Integer = surface_mesh.Positions.Count
                Dim y1 As Double = F(x, z)
                surface_mesh.Positions.Add(New Point3D(x, y1, z))
                surface_mesh.TextureCoordinates.Add(New Point(0, (y1 - ymin) / dy))

                Dim i2 As Integer = surface_mesh.Positions.Count
                Dim y2 As Double = F(x, z + dz)
                surface_mesh.Positions.Add(New Point3D(x, y2, z + dz))
                surface_mesh.TextureCoordinates.Add(New Point(0, (y2 - ymin) / dy))

                Dim i3 As Integer = surface_mesh.Positions.Count
                Dim y3 As Double = F(x + dx, z + dz)
                surface_mesh.Positions.Add(New Point3D(x + dx, y3, z + dz))
                surface_mesh.TextureCoordinates.Add(New Point(0, (y3 - ymin) / dy))

                Dim i4 As Integer = surface_mesh.Positions.Count
                Dim y4 As Double = F(x + dx, z)
                surface_mesh.Positions.Add(New Point3D(x + dx, y4, z))
                surface_mesh.TextureCoordinates.Add(New Point(0, (y4 - ymin) / dy))

                ' Make the triangles.
                surface_mesh.TriangleIndices.Add(i1)
                surface_mesh.TriangleIndices.Add(i2)
                surface_mesh.TriangleIndices.Add(i3)

                surface_mesh.TriangleIndices.Add(i1)
                surface_mesh.TriangleIndices.Add(i3)
                surface_mesh.TriangleIndices.Add(i4)

                z += dz
            Next iz
            x += dx
        Next ix
    End Sub

    Private Function SinXZ(ByVal x As Double, ByVal z As Double) As Double
        Return Sin(x * z) / 2
    End Function

    Private Function SinXplusSinZ(ByVal x As Double, ByVal z As Double) As Double
        Return (Sin(x * 2) + Sin(z * 2)) / 2
    End Function

    Private Function Splash(ByVal x As Double, ByVal z As Double) As Double
        Dim r As Double = x * x + z * z
        Return Cos(r) / (1 + r / 2)
    End Function

    ' Make a horizontal label at the specified position.
    Private Sub MakeTitle(ByVal the_model As Model3DGroup)
        Dim lbl_sty As New Style(GetType(Label))
        lbl_sty.Setters.Add(New Setter(Label.ForegroundProperty, Brushes.Red))
        lbl_sty.Setters.Add(New Setter(Label.FontFamilyProperty, New FontFamily("Courier New")))
        lbl_sty.Setters.Add(New Setter(Label.FontWeightProperty, FontWeights.Bold))

        Dim sp As New StackPanel()
        sp.Background = Brushes.Transparent
        sp.Orientation = Orientation.Horizontal
        Dim normal As New Thickness(0, 0, 0, 0)
        Dim backup As New Thickness(-5, 0, 0, 0)
        Dim super As New Thickness(-5, -3, 0, 0)
        sp.Children.Add(New Label() With {.Style = lbl_sty, .Margin = normal, .Content = "Cos(x"})
        sp.Children.Add(New Label() With {.Style = lbl_sty, .Margin = super, .Content = "2"})
        sp.Children.Add(New Label() With {.Style = lbl_sty, .Margin = normal, .Content = "+ z"})
        sp.Children.Add(New Label() With {.Style = lbl_sty, .Margin = super, .Content = "2"})
        sp.Children.Add(New Label() With {.Style = lbl_sty, .Margin = backup, .Content = ") / [1 + (x"})
        sp.Children.Add(New Label() With {.Style = lbl_sty, .Margin = super, .Content = "2"})
        sp.Children.Add(New Label() With {.Style = lbl_sty, .Margin = normal, .Content = "+ z"})
        sp.Children.Add(New Label() With {.Style = lbl_sty, .Margin = super, .Content = "2"})
        sp.Children.Add(New Label() With {.Style = lbl_sty, .Margin = backup, .Content = ") / 2]"})

        Dim label_brush As New VisualBrush(sp)
        Dim label_material As New DiffuseMaterial(label_brush)
        Dim label_mesh As MeshGeometry3D = Nothing
        MakeRectangle(the_model, label_mesh, label_material, _
            New Point3D(-3, 3, 4), New Point3D(-3, 1, 4), _
            New Point3D(-3, 1, -4), New Point3D(-3, 3, -4), _
            New Point(0, 0), New Point(0, 1), New Point(1, 1), New Point(1, 0))
    End Sub

    ' Make a rectangle. If rect_mesh is Nothing, make a new one and add it to the model.
    ' The points p1, p2, p3, p4 should be outwardly oriented.
    ' The points u1, u2, u3, u4 give the texture coordinates for the points p1, p2, p3, p4.
    Private Sub MakeRectangle(ByVal model_group As Model3DGroup, _
     ByRef rect_mesh As MeshGeometry3D, ByVal rect_material As Material, _
     ByVal p1 As Point3D, ByVal p2 As Point3D, ByVal p3 As Point3D, ByVal p4 As Point3D, _
     ByVal u1 As Point, ByVal u2 As Point, ByVal u3 As Point, ByVal u4 As Point)

        ' Make the mesh if we must.
        If rect_mesh Is Nothing Then
            rect_mesh = New MeshGeometry3D()
            Dim new_model As New GeometryModel3D(rect_mesh, rect_material)
            model_group.Children.Add(new_model)
        End If

        ' Make the points.
        rect_mesh.Positions.Add(p1)
        rect_mesh.Positions.Add(p2)
        rect_mesh.Positions.Add(p3)
        rect_mesh.Positions.Add(p4)

        ' Set the texture coordinates.
        rect_mesh.TextureCoordinates.Add(u1)
        rect_mesh.TextureCoordinates.Add(u2)
        rect_mesh.TextureCoordinates.Add(u3)
        rect_mesh.TextureCoordinates.Add(u4)

        ' Make the triangles.
        Dim i1 As Integer = rect_mesh.Positions.Count - 4
        rect_mesh.TriangleIndices.Add(i1)
        rect_mesh.TriangleIndices.Add(i1 + 1)
        rect_mesh.TriangleIndices.Add(i1 + 2)

        rect_mesh.TriangleIndices.Add(i1)
        rect_mesh.TriangleIndices.Add(i1 + 2)
        rect_mesh.TriangleIndices.Add(i1 + 3)
    End Sub
End Class
