Imports System
Imports System.IO
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Navigation

Imports System.Math
Imports System.Windows.Media.Media3D

Partial Public Class Window1
    Public Sub New()
        MyBase.New()

        Me.InitializeComponent()

        ' Insert code required on object creation below this point.
    End Sub

    ' Build the model.
    Private Sub Window_Loaded(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)
        MakeSingleMeshSphere(Sphere00, New DiffuseMaterial(Brushes.Green), 1, 20, 30)
        MakeSingleMeshSphere(Sphere01, New SpecularMaterial(Brushes.Green, 50), 1, 20, 30)
        MakeSingleMeshSphere(Sphere02, New EmissiveMaterial(Brushes.DarkGreen), 1, 20, 30)

        Dim combined_material As New MaterialGroup()
        combined_material.Children.Add(New DiffuseMaterial(Brushes.Green))
        combined_material.Children.Add(New SpecularMaterial(Brushes.Green, 50))
        combined_material.Children.Add(New EmissiveMaterial(Brushes.DarkGreen))
        MakeSingleMeshSphere(Sphere03, combined_material, 1, 20, 30)
    End Sub

    ' Make a sphere.
    Private Sub MakeSingleMeshSphere(ByVal model_group As Model3DGroup, ByVal sphere_material As Material, ByVal radius As Double, ByVal num_phi As Integer, ByVal num_theta As Integer)
        ' Define the new mesh geometry.
        Dim new_mesh As New MeshGeometry3D()
        new_mesh.Positions = New Point3DCollection(3)
        new_mesh.TriangleIndices = New Int32Collection(3)

        Dim dphi As Double = PI / num_phi
        Dim dtheta As Double = 2 * PI / num_theta
        Dim theta As Double = 0

        ' Make the top point.
        new_mesh.Positions.Add(New Point3D(0, radius, 0))

        ' Make the middle points.
        Dim phi1 As Double = PI / 2 - dphi
        For p As Integer = 1 To num_phi - 1
            Dim r1 = radius * Cos(phi1)
            Dim y1 = radius * Sin(phi1)

            theta = 0
            For t As Integer = 1 To num_theta
                new_mesh.Positions.Add(New Point3D(r1 * Cos(theta), y1, -r1 * Sin(theta)))
                theta += dtheta
            Next t
            phi1 -= dphi
        Next p

        ' Make the bottom point.
        new_mesh.Positions.Add(New Point3D(0, -radius, 0))

        ' Make the triangles.
        ' Make the top fan.
        Dim i1, i2, i3, i4 As Integer
        i1 = num_theta
        i2 = 1
        For i As Integer = 1 To num_theta
            new_mesh.TriangleIndices.Add(0)
            new_mesh.TriangleIndices.Add(i1)
            new_mesh.TriangleIndices.Add(i2)
            i1 = i2
            i2 += 1
        Next i

        ' Make the middle triangles.
        For p As Integer = 0 To num_phi - 3
            i3 = 1 + p * num_theta
            i4 = i3 + num_theta
            i1 = i4 - 1
            i2 = i1 + num_theta
            For t As Integer = 0 To num_theta - 1
                new_mesh.TriangleIndices.Add(i1)
                new_mesh.TriangleIndices.Add(i2)
                new_mesh.TriangleIndices.Add(i4)

                new_mesh.TriangleIndices.Add(i1)
                new_mesh.TriangleIndices.Add(i4)
                new_mesh.TriangleIndices.Add(i3)

                i1 = i3
                i2 = i4
                i3 += 1
                i4 += 1
            Next t
        Next p

        ' Make the bottom fan.
        i3 = new_mesh.Positions.Count - 1 ' The last (bottom) point.
        i1 = i3 - 1
        i2 = i3 - num_theta
        For i As Integer = 1 To num_theta
            new_mesh.TriangleIndices.Add(i1)
            new_mesh.TriangleIndices.Add(i3)
            new_mesh.TriangleIndices.Add(i2)
            i1 = i2
            i2 += 1
        Next i

        ' Set the model's material.
        Dim new_model As New GeometryModel3D(new_mesh, sphere_material)

        ' Add the new model to the model group.
        model_group.Children.Add(new_model)
    End Sub
End Class
