﻿Imports System
Imports System.IO
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Navigation

Imports System.Windows.Media.Media3D
Imports System.Windows.Media.Imaging
Imports System.Math

Partial Public Class Window1
	Public Sub New()
		MyBase.New()

		Me.InitializeComponent()

		' Insert code required on object creation below this point.
	End Sub

    ' Build the scene.
    Private Sub Window_Loaded(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)
        ' Ground.
        Dim ground_brush As New ImageBrush(New BitmapImage(New Uri("pack://application:,,,/grass_and_brick.jpg")))
        Dim ground_material As Material = New DiffuseMaterial(ground_brush)
        Dim ground_mesh As MeshGeometry3D = Nothing
        MakeRectangle(TheModel, ground_mesh, ground_material, _
            New Point3D(-5, 0, -5), New Point3D(-5, 0, 5), New Point3D(5, 0, 5), New Point3D(5, 0, -5), _
            New Point(0, 0), New Point(0, 1), New Point(1, 1), New Point(1, 0))

        ' Rock boxes.
        Dim rock_brush As New ImageBrush(New BitmapImage(New Uri("pack://application:,,,/rock.jpg")))
        Dim rock_material As Material = New DiffuseMaterial(rock_brush)
        Dim rock_mesh As MeshGeometry3D = Nothing
        For x As Integer = -4 To 2 Step 6
            For z As Integer = -4 To 2 Step 6
                MakeBox(TheModel, rock_mesh, rock_material, _
                    New Point3D(x + 0.5, 1.5, z + 0.5), New Point3D(x + 0.5, 1.5, z + 1.5), New Point3D(x + 1.5, 1.5, z + 1.5), New Point3D(x + 1.5, 1.5, z + 0.5), _
                    New Point3D(x, 0, z), New Point3D(x, 0, z + 2), New Point3D(x + 2, 0, z + 2), New Point3D(x + 2, 0, z), _
                    New Point(0, 0), New Point(0, 1), New Point(1, 1), New Point(1, 0))
            Next z
        Next x

        ' Metal boxes.
        Dim metal_brush As New ImageBrush(New BitmapImage(New Uri("pack://application:,,,/metal.jpg")))
        Dim metal_material As Material = New DiffuseMaterial(metal_brush)
        Dim metal_mesh As MeshGeometry3D = Nothing
        For x As Integer = -4 To 2 Step 3
            For z As Integer = -4 To 2 Step 3
                If (x + z) Mod 2 <> 0 Then
                    MakeBox(TheModel, metal_mesh, metal_material, _
                        New Point3D(x, 2, z), New Point3D(x, 2, z + 2), New Point3D(x + 2, 2, z + 2), New Point3D(x + 2, 2, z), _
                        New Point3D(x, 0, z), New Point3D(x, 0, z + 2), New Point3D(x + 2, 0, z + 2), New Point3D(x + 2, 0, z), _
                        New Point(0, 0), New Point(0, 1), New Point(1, 1), New Point(1, 0))
                End If
            Next z
        Next x

        ' Cloud cone.
        Dim cloud_brush As New ImageBrush(New BitmapImage(New Uri("pack://application:,,,/clouds.jpg")))
        Dim cloud_material As Material = New DiffuseMaterial(cloud_brush)
        Dim cloud_mesh As MeshGeometry3D = Nothing
        MakeCylinder(TheModel, cloud_mesh, cloud_material, 0, 0, 0, 0, 4, 0, 0, 0.2, 0.8, 1, 1.25, 0.5, 30)

        ' Globe.
        Dim globe_brush As New ImageBrush(New BitmapImage(New Uri("pack://application:,,,/world.jpg")))
        Dim globe_material As Material = New DiffuseMaterial(globe_brush)
        Dim globe_mesh As MeshGeometry3D = Nothing
        MakeSphere(TheModel, globe_mesh, globe_material, 1, 0, 3.75, 0, 20, 30)
    End Sub

    ' Make a rectangle. If rect_mesh is Nothing, make a new one and add it to the model.
    ' The points p1, p2, p3, p4 should be outwardly oriented.
    ' The points u1, u2, u3, u4 give the texture coordinates
	' for the points p1, p2, p3, p4.
    Private Sub MakeRectangle(ByVal model_group As Model3DGroup, _
     ByRef rect_mesh As MeshGeometry3D, ByVal rect_material As Material, _
     ByVal p1 As Point3D, ByVal p2 As Point3D, ByVal p3 As Point3D, ByVal p4 As Point3D, _
     ByVal u1 As Point, ByVal u2 As Point, ByVal u3 As Point, ByVal u4 As Point)

        ' Make the mesh if we must.
        If rect_mesh Is Nothing Then
            rect_mesh = New MeshGeometry3D()
            Dim new_model As New GeometryModel3D(rect_mesh, rect_material)
            model_group.Children.Add(new_model)
        End If

        ' Make the points.
        rect_mesh.Positions.Add(p1)
        rect_mesh.Positions.Add(p2)
        rect_mesh.Positions.Add(p3)
        rect_mesh.Positions.Add(p4)

        ' Set the texture coordinates.
        rect_mesh.TextureCoordinates.Add(u1)
        rect_mesh.TextureCoordinates.Add(u2)
        rect_mesh.TextureCoordinates.Add(u3)
        rect_mesh.TextureCoordinates.Add(u4)

        ' Make the triangles.
        Dim i1 As Integer = rect_mesh.Positions.Count - 4
        rect_mesh.TriangleIndices.Add(i1)
        rect_mesh.TriangleIndices.Add(i1 + 1)
        rect_mesh.TriangleIndices.Add(i1 + 2)

        rect_mesh.TriangleIndices.Add(i1)
        rect_mesh.TriangleIndices.Add(i1 + 2)
        rect_mesh.TriangleIndices.Add(i1 + 3)
    End Sub

    ' Make a cube. The points p1, p2, p3, p4 should be outwardly oriented.
    ' The points p5, p6, p7, p8 should be opposite p1, p2, p3, p4 (so they are inwardly oriented).
    ' The points u1, u2, u3, u4 map to the box's sides.
    Private Sub MakeBox(ByVal box_model As Model3DGroup, _
     ByRef box_mesh As MeshGeometry3D, ByVal box_material As Material, _
     ByVal p1 As Point3D, ByVal p2 As Point3D, ByVal p3 As Point3D, ByVal p4 As Point3D, _
     ByVal p5 As Point3D, ByVal p6 As Point3D, ByVal p7 As Point3D, ByVal p8 As Point3D, _
     ByVal u1 As Point, ByVal u2 As Point, ByVal u3 As Point, ByVal u4 As Point)

        ' Top.
        MakeRectangle(box_model, box_mesh, box_material, p1, p2, p3, p4, u1, u2, u3, u4)
        ' Bottom.
        MakeRectangle(box_model, box_mesh, box_material, p8, p7, p6, p5, u1, u2, u3, u4)
        ' Left.
        MakeRectangle(box_model, box_mesh, box_material, p1, p5, p6, p2, u1, u2, u3, u4)
        ' Right.
        MakeRectangle(box_model, box_mesh, box_material, p3, p7, p8, p4, u1, u2, u3, u4)
        ' Front.
        MakeRectangle(box_model, box_mesh, box_material, p2, p6, p7, p3, u1, u2, u3, u4)
        ' Back.
        MakeRectangle(box_model, box_mesh, box_material, p4, p8, p5, p1, u1, u2, u3, u4)
    End Sub

    ' Make a sphere.
    Private Sub MakeSphere(ByVal model_group As Model3DGroup, ByRef sphere_mesh As MeshGeometry3D, ByVal sphere_material As Material, _
     ByVal radius As Double, ByVal cx As Double, ByVal cy As Double, ByVal cz As Double, _
     ByVal num_phi As Integer, ByVal num_theta As Integer)

        ' Make the mesh if we must.
        If sphere_mesh Is Nothing Then
            sphere_mesh = New MeshGeometry3D()
            Dim new_model As New GeometryModel3D(sphere_mesh, sphere_material)
            model_group.Children.Add(new_model)
        End If

        Dim dphi As Double = PI / num_phi
        Dim dtheta As Double = 2 * PI / num_theta

        ' Remember the first point.
        Dim pt0 As Integer = sphere_mesh.Positions.Count

        ' Make the points.
        Dim phi1 As Double = PI / 2
        For p As Integer = 0 To num_phi
            Dim r1 As Double = radius * Cos(phi1)
            Dim y1 As Double = radius * Sin(phi1)

            Dim theta As Double = 0
            For t As Integer = 0 To num_theta
                sphere_mesh.Positions.Add(New Point3D( _
                    cx + r1 * Cos(theta), cy + y1, cz + -r1 * Sin(theta)))
                sphere_mesh.TextureCoordinates.Add(New Point( _
                    t / num_theta, p / num_phi))
                theta += dtheta
            Next t
            phi1 -= dphi
        Next p

        ' Make the triangles.
        Dim i1, i2, i3, i4 As Integer
        For p As Integer = 0 To num_phi - 1
            i1 = p * (num_theta + 1)
            i2 = i1 + (num_theta + 1)
            For t As Integer = 0 To num_theta - 1
                i3 = i1 + 1
                i4 = i2 + 1
                sphere_mesh.TriangleIndices.Add(pt0 + i1)
                sphere_mesh.TriangleIndices.Add(pt0 + i2)
                sphere_mesh.TriangleIndices.Add(pt0 + i4)

                sphere_mesh.TriangleIndices.Add(pt0 + i1)
                sphere_mesh.TriangleIndices.Add(pt0 + i4)
                sphere_mesh.TriangleIndices.Add(pt0 + i3)
                i1 += 1
                i2 += 1
            Next t
        Next p
    End Sub

    ' Make a cylinder using vectors perpendicular to the axis.
    Private Sub MakeCylinder(ByVal model_group As Model3DGroup, _
     ByRef cylinder_mesh As MeshGeometry3D, _
     ByVal cylinder_material As Material, _
     ByVal cx1 As Double, ByVal cy1 As Double, ByVal cz1 As Double, _
     ByVal cx2 As Double, ByVal cy2 As Double, ByVal cz2 As Double, _
     ByVal uv1 As Double, ByVal uv2 As Double, ByVal uv3 As Double, ByVal uv4 As Double, _
     ByVal radius1 As Double, ByVal radius2 As Double, ByVal num_theta As Integer)

        ' Get the axis.
        Dim axis As New Vector3D(cx2 - cx1, cy2 - cy1, cz2 - cz1)

        ' Just pick a vector to start from.
        Dim vector1 As New Vector3D(1, 0, 0)
        If Vector3D.AngleBetween(axis, vector1) < 0.1 Then ' See if they're parallel.
            ' Pick a different vector1.
            vector1 = New Vector3D(0, 0, 1)
        End If

        ' Find perpendicular vectors.
        Dim vector2 As Vector3D = Vector3D.CrossProduct(axis, vector1)
        vector1 = Vector3D.CrossProduct(axis, vector2)
        vector1.Normalize()
        vector2.Normalize()

        ' Make the cylinder.
        MakeCylinder(model_group, cylinder_mesh, cylinder_material, _
            cx1, cy1, cz1, cx2, cy2, cz2, uv1, uv2, uv3, uv4, _
            radius1, radius2, num_theta, vector1, vector2)
    End Sub

    ' Make a cylinder. (cx1, cy2, cz1) and (cx2, cy2, cz2) are the centers of the ends.
    ' vector1 and vector2 are non-parallel vectors used to build the ends. If these are missing,
    ' the routine uses perpendicular unit vectors. Normally these should be unit vectors or omitted.
    Private Sub MakeCylinder(ByVal model_group As Model3DGroup, _
     ByRef cylinder_mesh As MeshGeometry3D, _
     ByVal cylinder_material As Material, _
     ByVal cx1 As Double, ByVal cy1 As Double, ByVal cz1 As Double, _
     ByVal cx2 As Double, ByVal cy2 As Double, ByVal cz2 As Double, _
     ByVal uv1 As Double, ByVal uv2 As Double, ByVal uv3 As Double, ByVal uv4 As Double, _
     ByVal radius1 As Double, ByVal radius2 As Double, ByVal num_theta As Integer, _
     ByVal vector1 As Vector3D, ByVal vector2 As Vector3D)

        ' Make the mesh if we must.
        If cylinder_mesh Is Nothing Then
            cylinder_mesh = New MeshGeometry3D()
            Dim cylinder_model As New GeometryModel3D(cylinder_mesh, cylinder_material)
            model_group.Children.Add(cylinder_model)
        End If

        ' Remember the index of the first point in this cylinder.
        Dim pt0 As Integer = cylinder_mesh.Positions.Count

        ' Make the top center point.
        cylinder_mesh.Positions.Add(New Point3D(cx1, cy1, cz1))
        cylinder_mesh.TextureCoordinates.Add(New Point(0.5, uv1))

        ' Make the top edge points.
        Dim dtheta As Double = 2 * PI / num_theta
        Dim theta As Double = 0
        For t As Integer = 0 To num_theta
            Dim x As Double = cx1 + radius1 * vector1.X * Cos(theta) + radius1 * vector2.X * Sin(theta)
            Dim y As Double = cy1 + radius1 * vector1.Y * Cos(theta) + radius1 * vector2.Y * Sin(theta)
            Dim z As Double = cz1 + radius1 * vector1.Z * Cos(theta) + radius1 * vector2.Z * Sin(theta)
            cylinder_mesh.Positions.Add(New Point3D(x, y, z))
            cylinder_mesh.TextureCoordinates.Add(New Point(t / num_theta, uv2))
            theta += dtheta
        Next t

        ' Make the bottom edge points.
        theta = 0
        For t As Integer = 0 To num_theta
            Dim x As Double = cx2 + radius2 * vector1.X * Cos(theta) + radius2 * vector2.X * Sin(theta)
            Dim y As Double = cy2 + radius2 * vector1.Y * Cos(theta) + radius2 * vector2.Y * Sin(theta)
            Dim z As Double = cz2 + radius2 * vector1.Z * Cos(theta) + radius2 * vector2.Z * Sin(theta)

            cylinder_mesh.Positions.Add(New Point3D(x, y, z))
            cylinder_mesh.TextureCoordinates.Add(New Point(t / num_theta, uv3))
            theta += dtheta
        Next t

        ' Make the bottom center point.
        cylinder_mesh.Positions.Add(New Point3D(cx2, cy2, cz2))
        cylinder_mesh.TextureCoordinates.Add(New Point(0.5, uv4))

        ' Make the triangles.
        Dim i1, i2, i3, i4, i_bottom As Integer
        i_bottom = cylinder_mesh.Positions.Count - 1
        i1 = 1
        i2 = i1 + (num_theta + 1)
        For t As Integer = 0 To num_theta - 1
            i3 = i1 + 1
            i4 = i2 + 1

            ' Top.
            cylinder_mesh.TriangleIndices.Add(pt0 + 0)
            cylinder_mesh.TriangleIndices.Add(pt0 + i1)
            cylinder_mesh.TriangleIndices.Add(pt0 + i3)

            ' Side.
            cylinder_mesh.TriangleIndices.Add(pt0 + i1)
            cylinder_mesh.TriangleIndices.Add(pt0 + i2)
            cylinder_mesh.TriangleIndices.Add(pt0 + i4)

            cylinder_mesh.TriangleIndices.Add(pt0 + i1)
            cylinder_mesh.TriangleIndices.Add(pt0 + i4)
            cylinder_mesh.TriangleIndices.Add(pt0 + i3)

            ' Bottom.
            cylinder_mesh.TriangleIndices.Add(i_bottom)
            cylinder_mesh.TriangleIndices.Add(pt0 + i4)
            cylinder_mesh.TriangleIndices.Add(pt0 + i2)

            i1 += 1
            i2 += 1
        Next t
    End Sub
End Class
