Imports System
Imports System.IO
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Navigation

Imports System.Math
Imports System.Windows.Media.Media3D

Partial Public Class Window1
    Public Sub New()
        MyBase.New()

        Me.InitializeComponent()

        ' Insert code required on object creation below this point.
    End Sub

    ' Build the model.
    Private Sub Window_Loaded(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)
        MakeSphere(Sphere00, New DiffuseMaterial(Brushes.White), 1, 6, 10, False)
        MakeSphere(Sphere01, New DiffuseMaterial(Brushes.White), 1, 6, 10, True)

        MakeSphere(Sphere10, New DiffuseMaterial(Brushes.White), 1, 20, 30, False)
        MakeSphere(Sphere11, New DiffuseMaterial(Brushes.White), 1, 20, 30, True)
    End Sub

    ' Make a sphere.
    Private Sub MakeSphere(ByVal model_group As Model3DGroup, ByVal sphere_material As Material, ByVal radius As Double, ByVal num_phi As Integer, ByVal num_theta As Integer, ByVal use_radius_normals As Boolean)
        Dim dphi As Double = PI / num_phi
        Dim dtheta As Double = 2 * PI / num_theta
        Dim phi As Double = -PI / 2
        Dim theta As Double = 0
        Dim r, y As Double
        Dim p1, p2, p3, p4 As Point3D

        ' Make the top triangle fan.
        phi = PI / 2 - dphi
        r = radius * Cos(phi)
        p1 = New Point3D(0, radius, 0)
        y = radius * Sin(phi)

        theta = dtheta
        p3 = New Point3D(r * Cos(theta), y, r * Sin(theta))
        theta = 0
        For i As Integer = 1 To num_theta
            p2 = p3
            p3 = New Point3D(r * Cos(theta), y, -r * Sin(theta))
            MakeTriangle(model_group, sphere_material, p1, p2, p3, use_radius_normals)
            theta += dtheta
        Next i

        ' Make the bottom triangle fan.
        phi = -PI / 2 + dphi
        theta = 0
        r = radius * Cos(phi)
        p1 = New Point3D(0, -radius, 0)
        y = radius * Sin(phi)

        theta = -dtheta
        p3 = New Point3D(r * Cos(theta), y, r * Sin(theta))
        theta = 0
        For i As Integer = 1 To num_theta
            p2 = p3
            p3 = New Point3D(r * Cos(theta), y, r * Sin(theta))
            MakeTriangle(model_group, sphere_material, p1, p2, p3, use_radius_normals)
            theta += dtheta
        Next i

        ' Make the strips between.
        Dim phi1 As Double = PI / 2 - dphi

        For p As Integer = 1 To num_phi - 2
            Dim phi2 As Double = phi1 - dphi
            Dim r1 = radius * Cos(phi1)
            Dim r2 = radius * Cos(phi2)

            Dim y1 = radius * Sin(phi1)
            Dim y2 = radius * Sin(phi2)

            theta = -dtheta
            p3 = New Point3D(r1 * Cos(theta), y1, -r1 * Sin(theta))
            p4 = New Point3D(r2 * Cos(theta), y2, -r2 * Sin(theta))
            theta = 0
            For t As Integer = 1 To num_theta
                p1 = p3
                p2 = p4
                p3 = New Point3D(r1 * Cos(theta), y1, -r1 * Sin(theta))
                p4 = New Point3D(r2 * Cos(theta), y2, -r2 * Sin(theta))
                MakeTriangle(model_group, sphere_material, p1, p2, p4, use_radius_normals)
                MakeTriangle(model_group, sphere_material, p1, p4, p3, use_radius_normals)
                theta += dtheta
            Next t
            phi1 = phi2
        Next p
    End Sub

    ' Make a single triangle.
    Private Sub MakeTriangle(ByVal model_group As Model3DGroup, ByVal triangle_material As Material, ByVal p1 As Point3D, ByVal p2 As Point3D, ByVal p3 As Point3D, ByVal use_radius_normals As Boolean)
        ' Define the new mesh geometry.
        Dim new_mesh As New MeshGeometry3D()

        ' Define the points.
        new_mesh.Positions = New Point3DCollection(3)
        new_mesh.Positions.Add(p1)
        new_mesh.Positions.Add(p2)
        new_mesh.Positions.Add(p3)

        ' Define the triangle.
        new_mesh.TriangleIndices = New Int32Collection(3)
        new_mesh.TriangleIndices.Add(0)
        new_mesh.TriangleIndices.Add(1)
        new_mesh.TriangleIndices.Add(2)

        ' Set normals if appropriate.
        If use_radius_normals Then
            new_mesh.Normals = New Vector3DCollection(3)
            new_mesh.Normals.Add(New Vector3D(p1.X, p1.Y, p1.Z))
            new_mesh.Normals.Add(New Vector3D(p2.X, p2.Y, p2.Z))
            new_mesh.Normals.Add(New Vector3D(p3.X, p3.Y, p3.Z))
        End If

        ' Set the model's material.
        Dim new_model As New GeometryModel3D(new_mesh, triangle_material)

        ' Add the new model to the model group.
        model_group.Children.Add(new_model)
    End Sub
End Class
